package kcode.statexmi2java;

import java.util.prefs.Preferences;

import kcode.statexmi2java.gui.Wizard;
import kcode.statexmi2java.xmireader.IXMIReader;
import kcode.statexmi2java.xmireader.XMIReaderJAXB;
import kcode.statexmi2java.codegenerator.CodeGenerator;
import kcode.statexmi2java.codegenerator.JavaGenerator;
import kcode.statexmi2java.codegenerator.templates.TemplatorFreeMarker;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class StateXMI2Java {

  /**
   * sub-path for accessing template files (as resources via classloader)
   */
  public static final String TEMPLATE_SUBDIR = "/templates";

  protected final Logger     logger          = LoggerFactory.getLogger(StateXMI2Java.class);

  public static void main(String[] args) {
    StateXMI2Java mainObject = new StateXMI2Java();
    mainObject.runWizard();
  }

  /**
   * setup and start the wizard
   */
  public void runWizard() {
    // Initialize swing to native look and feel
    try {
      javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
    } catch (ClassNotFoundException e) {
      logger.error("ClassNotFoundException on runWizard {}", e.getMessage());
    } catch (InstantiationException e) {
      logger.error("InstantiationException on runWizard {}", e.getMessage());
    } catch (IllegalAccessException e) {
      logger.error("IllegalAccessException on runWizard {}", e.getMessage());
    } catch (javax.swing.UnsupportedLookAndFeelException e) {
      logger.error("UnsupportedLookAndFeelException on runWizard {}", e.getMessage());
    }

    // Instantiate and display the wizard
    IXMIReader xmiReader = new XMIReaderJAXB();
    CodeGenerator codeGenerator = new JavaGenerator(new TemplatorFreeMarker(this.getClass(),
        TEMPLATE_SUBDIR), this.getClass(), TEMPLATE_SUBDIR);
    Wizard wizard = new Wizard(xmiReader, codeGenerator, Preferences.userNodeForPackage(kcode.statexmi2java.StateXMI2Java.class));
    wizard.display();
  }
}
