package kcode.statexmi2java.codegenerator.templates;

import java.util.List;

/**
 * A templator to parse templates and return code
 */
public interface ITemplator {

  /**
   * Get code for a transition
   * @param classname name of the transition class to use
   * @param guard the guard condition as String or null if no guard
   * @param effectCode The transitions effect code or null if no effect
   * @return
   */
  public String getTransitionCode(String classname, String guard, String effectCode);
  /**
   * get code for a state with a specified state name
   * @param stateName the name of the state
   * @return code of the state class as string
   */
  public String getStateCode(String stateName);
  /**
   * get code for a state with a specified state name
   * @param stateName the name of the state
   * @param classinner inner code of the state class 
   * @return code of the state class as string
   */
  public String getStateCode(String stateName, String classinner);
  /**
   * get enum code for trigger
   * @param triggers list of triggers
   * @return code of the enum class as string
   */
  public String getTriggerEnumCode(List<String> triggers);
  /**
   * get enum code for stateID
   * @param stateClassNames list of state class names
   * @return code of the enum class as string
   */
  public String getStateIDEnumCode(List<String> stateClassNames);
  /**
   * get the code for a statefactory
   * @param createStatesCode code for creating states
   * @param createTransitionsCode code for creating transitions
   * @param initialStateID initial state name
   * @return code for state factory class
   */
  public String getStateFactoryCode(String createStatesCode, String createTransitionsCode, String initialStateID);
}
