package kcode.statexmi2java.gui;

import java.io.File;
import java.util.List;
import java.util.prefs.Preferences;

import javax.swing.JDialog;
import javax.swing.JFrame;

import kcode.statexmi2java.codegenerator.CodeGenerator;
import kcode.statexmi2java.xmireader.IXMIReader;

import org.ciscavate.cjwizard.PageFactory;
import org.ciscavate.cjwizard.WizardContainer;
import org.ciscavate.cjwizard.WizardListener;
import org.ciscavate.cjwizard.WizardPage;
import org.ciscavate.cjwizard.WizardSettings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Wizard {

  // keys for settings (wizard data model)
  public static final String  SETT_KEY_XMI           = "xmiFilePath";
  public static final String  SETT_KEY_GENTARGET     = "targetFolderPath";
  public static final String  SETT_KEY_STATEMACHINES = "statemachines";
  public static final String  SETT_KEY_DIAGRAMSEL    = "smSelection";
  // preferences
  private static final String PREF_KEY_LASTXMIPATH   = "lastxmipath";
  private static final String PREF_KEY_LASTGENPATH   = "lastgenpath";
  public static Preferences   prefNode;
  // frame title prefix
  protected static final String      TITLE_PREFIX            = "StateXMI2Java";

  protected final Logger      logger                 = LoggerFactory.getLogger(WizardPageFactory.class);
  // frame containing UI
  protected final JFrame      mainFrame              = new JFrame();

  /**
   * GUI wizard
   * @param xmiReader xmi reader
   * @param codeGen code generator
   * @param preferencesNode preferences node
   */
  public Wizard(IXMIReader xmiReader, CodeGenerator codeGen, Preferences preferencesNode) {
    super();
    prefNode = preferencesNode;
    
    final PageFactory pageFactory = new WizardPageFactory(xmiReader, codeGen);
    final WizardContainer wizard = new WizardContainer(pageFactory);

    wizard.addWizardListener(new WizardListener() {
      @Override
      public void onCanceled(List<WizardPage> path, WizardSettings settings) {
        mainFrame.dispose();
      }

      @Override
      public void onFinished(List<WizardPage> path, WizardSettings settings) {
        Wizard.savePersistentPathForXMI((String) settings.get(SETT_KEY_XMI));
        Wizard.savePersistentPathForGen((String) settings.get(SETT_KEY_GENTARGET));
        logger.debug("wizard finished with path {} and settings {}", path, settings);
        mainFrame.dispose();
      }

      @Override
      public void onPageChanged(WizardPage newPage, List<WizardPage> path) {
        mainFrame.setTitle(TITLE_PREFIX + " – " + newPage.getDescription());
      }
    });

    // put the WizardContainer into the frame:
    mainFrame.getContentPane().add(wizard);
    mainFrame.pack();
    mainFrame.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
  }

  /**
   * display wizard
   */
  public void display() {
    mainFrame.setVisible(true);
  }

  /**
   * Get the persisted path for XMI, which was used as an XMI source file previously
   * 
   * @return String (potential) path to an XMI file
   */
  public static String getPersistentPathForXMI() {
    return prefNode.get(PREF_KEY_LASTXMIPATH, System.getProperty("user.home"));
  }

  /**
   * get the persisted path, which was used as a target folder previously
   * 
   * @return String (potential) path to a folder
   */
  public static String getPersistentPathForGen() {
    return prefNode.get(PREF_KEY_LASTGENPATH, System.getProperty("user.home" + File.separator + "gen"));
  }

  /**
   * Persist the path to an XMI file which was used as the source XMI file
   * 
   * @param path The path to the XMI file
   */
  public static void savePersistentPathForXMI(String path) {
    prefNode.put(PREF_KEY_LASTXMIPATH, path);
  }

  /**
   * Persist the path to a folder which was used as a target folder
   * 
   * @param path The Path to the target folder
   */
  public static void savePersistentPathForGen(String path) {
    prefNode.put(PREF_KEY_LASTGENPATH, path);
  }
}
