package kcode.statexmi2java.gui;

import java.awt.Component;
import java.util.List;

import javax.swing.DefaultComboBoxModel;
import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import kcode.statexmi2java.smrepresentation.State;
import kcode.statexmi2java.smrepresentation.StateMachine;

import org.ciscavate.cjwizard.WizardPage;
import org.ciscavate.cjwizard.WizardSettings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * CJWizard wizard page for state machine selection (select one from several)
 */
public class WizardPageDiagramSel extends WizardPage {
  protected static final String TEXT_LABEL_DIAGRAMCHOICE = "Diagrammwahl:";
  protected static final String TEXT_LABEL_MORETHANONESM = "In der gewählten XMI-Datei wurde mehr als ein Zustandsdiagramm gefunden.";
  // page constants
  public static final String WIZARDPAGE_TITLE          = "DiagramSel";
  public static final String WIZARDPAGE_DESC           = "Diagram Selection";

  private static final long  serialVersionUID          = 1;
  public static final String TEXT_SELECTIONENTRY_LABEL = "StateMachine with initial-state: ";

  protected final Logger     logger                    = LoggerFactory.getLogger(WizardPageFactory.class);
  protected JComboBox        diagramSel;
  protected WizardSettings   settings;

  public WizardPageDiagramSel(WizardSettings settings) {
    super(WIZARDPAGE_TITLE, WIZARDPAGE_DESC);
    this.settings = settings;
    initComponents();
  }

  @Override
  public void updateSettings(WizardSettings settings) {
    int index = diagramSel.getSelectedIndex();
    //TODO [REFACTOR] save SM (reference) instead of index
    // @SuppressWarnings("unchecked")
    // List<StateMachine> sMs = (List<StateMachine>) settings.get(Wizard.SETT_KEY_STATEMACHINES);
    // sMs.get(index);
    settings.put(Wizard.SETT_KEY_DIAGRAMSEL, index);
  }

  protected void initComponents() {
    // add labels
    add(new JLabel(TEXT_LABEL_MORETHANONESM));
    add(new JLabel(TEXT_LABEL_DIAGRAMCHOICE));
    
    // instantiate combobox for state machine selection
    @SuppressWarnings("unchecked")
    List<StateMachine> sMs = (List<StateMachine>) settings.get(Wizard.SETT_KEY_STATEMACHINES);
    DefaultComboBoxModel model = new DefaultComboBoxModel(sMs.toArray());
    diagramSel = new JComboBox(model);
    class CellRenderer extends BasicComboBoxRenderer {
      private static final long serialVersionUID = 1L;
      @Override
      public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
          boolean cellHasFocus) {
        Component ret = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
        if (value instanceof Icon) {
          setIcon((Icon) value);
        } else if (value instanceof StateMachine) {
          for (State state : ((StateMachine) value).states) {
            if (state.isInitial) {
              setText(WizardPageDiagramSel.TEXT_SELECTIONENTRY_LABEL + state.name);
              break;
            }
          }
        } else {
          setText((value == null) ? "" : value.toString());
        }
        return ret;
      }
    }
    diagramSel.setRenderer(new CellRenderer());
    add(diagramSel);
  }
}
