package codetemplates;

/**
 * state factory that instantiates the object tree for usage of the state machine this includes: states, transitions,
 * 
 * @param <S> StateIDs enum, identifying states
 * @param <T> Triggers enum, representing triggers
 */
public interface IStateFactory<S extends Enum<S>, T extends Enum<T>> {

  /**
   * get the state object to a state ID
   * @param id state ID
   * @return ObjectState
   */
  public ObjectState<S, T> getState(S id);

  /**
   * get initial state
   * @return ObjectState initial state
   */
  public ObjectState<S, T> getInitialState();
}
