package codetemplates;

import java.util.ArrayList;

/**
 * state representation
 * 
 * @param <S> StateID enum
 * @param <T> Trigger enum
 */
public class ObjectState<S extends Enum<S>, T extends Enum<T>> {
  private S                                  stateID;
  protected ArrayList<StateTransition<S, T>> transitions = new ArrayList<StateTransition<S, T>>();

  public ObjectState(S stateID) {
    this.stateID = stateID;
  }

  public final S getStateID() {
    return stateID;
  }

  public void addTransition(StateTransition<S, T> transition) {
    if (!this.transitions.contains(transition)) {
      this.transitions.add(transition);
    }
  }

  /**
   * do first applicable transition for a specified trigger
   * 
   * @param trigger the trigger to use
   * @param context context which can be used in guards etc for further environment specific checks
   * @return new state or null if no transition found for trigger
   */
  public ObjectState<S, T> triggerTransition(T trigger, Object context) {
    for (StateTransition<S, T> trans : transitions) {
      if (trans.isTriggerableBy(trigger, context)) {
        return trans.triggerTransitionBy(trigger, context);
      }
    }
    return null;
  }

  protected void onStateStay(Object context) {
    // call to onExit() & onEntry() may be applicable, but not always. Thus, default is nothing
  }

  protected void onStateEntry(Object context) {
    onStateDoActivity(context);
  }

  protected void onStateExit(Object context) {
    // nothing by default – to be overwritten by inheriting classes
  }

  protected void onStateDoActivity(Object context) {
  }
}
