package kcode.statexmi2java.codegenerator.templates;

import static org.junit.Assert.*;

import java.util.List;
import java.util.ArrayList;

import kcode.statexmi2java.StateXMI2Java;
import kcode.statexmi2java.codegenerator.templates.TemplatorFreeMarker;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class TemplatorFreeMarkerTest {
  
  static final protected String tmpDir = System.getProperty("java.io.tmpdir");

  protected TemplatorFreeMarker tes;
  
  @Before
  public void setUp() throws Exception {
    tes = new TemplatorFreeMarker(this.getClass(), StateXMI2Java.TEMPLATE_SUBDIR);
  }

  @After
  public void tearDown() throws Exception {
    tes = null;
  }

  @Test
  public void testGetCodeForTransition() {
    assertEquals("import codetemplates.*;\n\npublic class MyTransitionName<S extends Enum<S>, T extends Enum<T>> extends StateTransition<S, T> {\n  public MyTransitionName(ObjectState<S, T> source, ObjectState<S, T> target, T trigger) {\n    super(source, target, trigger);\n  }\n  \n}\n", tes.getTransitionCode("MyTransitionName", null, null));
  }
  @Test
  public void testGetCodeForTransitionWithGuard() {
    final String guard = "MaiGuard";
    final String guardCode = tes.getTransitionGuardCode(guard);
    final String expected = "import codetemplates.*;\n\npublic class MyTransitionName<S extends Enum<S>, T extends Enum<T>> extends StateTransition<S, T> {\n  public MyTransitionName(ObjectState<S, T> source, ObjectState<S, T> target, T trigger) {\n    super(source, target, trigger);\n  }\n  " + guardCode + "\n}\n";
    assertEquals(expected, tes.getTransitionCode("MyTransitionName", guard, null));
  }
  @Test
  public void testGetCodeForTransitionWithEffect() {
    final String effectC = "MaiEffaikt";
    final String expected = "import codetemplates.*;\n\npublic class MyTransitionName<S extends Enum<S>, T extends Enum<T>> extends StateTransition<S, T> {\n  public MyTransitionName(ObjectState<S, T> source, ObjectState<S, T> target, T trigger) {\n    super(source, target, trigger);\n  }\n  " + effectC + "\n}\n";
    assertEquals(expected, tes.getTransitionCode("MyTransitionName", null, effectC));
  }
  @Test
  public void testGetCodeForTransitionWithGuardAndEffect() {
    final String guard = "MaiGuard";
    final String guardCode = tes.getTransitionGuardCode(guard);
    final String effectC = "MaiEffaikt";
    final String expected = "import codetemplates.*;\n\npublic class MyTransitionName<S extends Enum<S>, T extends Enum<T>> extends StateTransition<S, T> {\n  public MyTransitionName(ObjectState<S, T> source, ObjectState<S, T> target, T trigger) {\n    super(source, target, trigger);\n  }\n  " + guardCode + effectC + "\n}\n";
    assertEquals(expected, tes.getTransitionCode("MyTransitionName", guard, effectC));
  }
  
  @Test
  public void testGetTransitionGuardCode() {
    assertEquals("  public boolean isGuardConditionMet(Object context) {\n    // TODO [gen] implement guard MaiGuard\n    return false;\n  }\n", tes.getTransitionGuardCode("MaiGuard"));
  }
  @Test
  public void testGetTransitionGuardCodeNull() {
    assertEquals("", tes.getTransitionGuardCode(null));
  }

  @Test
  public void testGetCodeForTriggers() {
    List<String> triggers = new ArrayList<String>();
    triggers.add("test");
    triggers.add("test2");
    triggers.add("testT");

    final String triggersCode = "TEST, TEST2, TESTT";
    final String expected = "\npublic enum Trigger {\n " + triggersCode + "\n}\n";
    assertEquals(expected, tes.getTriggerEnumCode(triggers));
  }
}
